// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "Connector.h"

#include "ClientSocket.h"

#include <poll.h>
#include <sys/socket.h>

//------------------------------------------------------------------------------

using hu::varadiistvan::scpl::io::Connector;

//------------------------------------------------------------------------------

bool Connector::connect()
{
    if (connected) return true;
    if (connecting || failed()) return false;

    size_t addrlen = 0;
    const struct sockaddr* addr = getAddress(addrlen);

    if (socket.connect(addr, addrlen)) {
        connected = true;
        return true;
    } else if (socket.failed()) {
        return false;
    } else {
        connecting = true;
        socket.events |= POLLOUT;
        return false;
    }
}

//------------------------------------------------------------------------------

void Connector::handleWritable()
{
    if (!connecting) return;

    connecting = false;
    socket.events &= ~POLLOUT;

    int error = 0;
    socklen_t len = sizeof(error);
    if (getsockopt(socket.fd, SOL_SOCKET, SO_ERROR, &error, &len)<0) {
        socket.setErrorCodeFromErrno();
    } else {
        if (error==0) {
            connected = true;
        } else {
            socket.setErrorCode(error);
        }
    }
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
